import ItemHTML from './ItemHTML.js';

export default class Item {

    static NAME_SELECTOR = '.item-link';
    static PRICE_SELECTOR = '.item-price';
    static GARAGE_SELECTOR = '.item-parking';

    static ROOM_SELECTOR = '.item-detail-char .item-detail:nth-child(1)';
    static METERS_SELECTOR = '.item-detail-char .item-detail:nth-child(2)';
    static ADDITIONAL_INFORMATION_SELECTOR = '.item-detail-char .item-detail:nth-child(3)';

    constructor(htmlNode) {
        this._node = htmlNode;

        this.name = this._extractName();

        this.locationName = this._extractLocationName();
        //this.location = this._extractLocation();

        this.price = this._extractPrice();
        this.meters = this._extractMeters();
        this.priceMeter = this._extractPriceMeter();

        this.additionalInfo = this._extractAdditionalInfo();
        this.hasGarage = this._extractGarage();
        this.hasLift = this._extractLift();
        this.isExterior = this._extractExterior();
        this.isInterior = this._extractInterior();
    }

    get _data() {
        return this._node.nextElementSibling;
    }

    refreshData() {
        this.removeData();
        this.addData();
    }

    isDataRendered() {
        const nextElement = this._data;
        return nextElement.className
            && nextElement.className.includes(ItemHTML.CONTAINER_CLASS_NAME);
    }

    isProperPrice(price) {
        return this.price <= price;
    }

    isProperPriceMeter(priceMeter) {
        return this.priceMeter <= priceMeter;
    }

    removeData() {
        if (!this.isDataRendered()) return;
        this._data.remove();
    }

    addData() {
        this._node.insertAdjacentHTML('afterend', ItemHTML.createInformation(this));
    }

    isFlat() {
        return this._nameIncludes('Piso');
    }

    isHouse() {
        return this._nameIncludes('Casa')
            || this._nameIncludes('Chalet')
            || this._nameIncludes('Finca');
    }

    // TODO: This may be included in isFlat.
    isGround() {
        return this._nameIncludes('Bajo');
    }

    _nameIncludes(name) {
        return this.name.toLowerCase().includes(name.toLowerCase());
    }

    _extractName() {
        return this._node.querySelector(Item.NAME_SELECTOR).textContent;
    }

    _extractLocationName() {
        return this.name
            .substring(this.name.lastIndexOf(','), this.name.length)
            .replaceAll('.', '').replaceAll(',', '');
    }

    _extractPrice() {
        const priceText = this._node.querySelector(Item.PRICE_SELECTOR).textContent;
        return Number(priceText.replace('€', '').replaceAll('.', '').replaceAll(',', ''));
    }

    _extractMeters() {
        let metersText = this._node.querySelector(Item.METERS_SELECTOR).textContent;
        if (!metersText.includes('m²')) {
            metersText = this._node.querySelector(Item.ROOM_SELECTOR).textContent;
        }
        return Number(metersText
            .replace('m²', '')
            .replaceAll('.', '')
        );
    }

    _extractPriceMeter() {
        return Math.round(this.price / this.meters);
    }

    _extractAdditionalInfo() {
        let additionalInfo = this._node.querySelector(Item.ADDITIONAL_INFORMATION_SELECTOR);
        if (!additionalInfo) {
            additionalInfo = this._node.querySelector(Item.METERS_SELECTOR);
        }
        return additionalInfo.textContent;
    }

    _extractLift() {
        if (!this.additionalInfo) return false;
        return this.additionalInfo.includes('con ascensor');
    }

    _extractExterior() {
        if (!this.additionalInfo) return false;
        return this.additionalInfo.includes('exterior');
    }

    _extractInterior() {
        if (!this.additionalInfo) return false;
        return this.additionalInfo.includes('interior');
    }

    _extractGarage() {
        return this._node.querySelector(Item.GARAGE_SELECTOR) !== null;
    }

}